@@ -31,14 +31,15 @@ import java.net.InetAddress;
 import java.util.Collection;
 
 import org.apache.http.HttpHost;
-import org.apache.http.annotation.Immutable;
+import org.apache.http.annotation.Contract;
+import org.apache.http.annotation.ThreadingBehavior;
 
 /**
  *  Immutable class encapsulating request configuration items.
  *  The default setting for stale connection checking changed
  *  to false, and the feature was deprecated starting with version 4.4.
  */
-@Immutable
+@Contract(threading = ThreadingBehavior.IMMUTABLE)
 public class RequestConfig implements Cloneable {
 
     public static final RequestConfig DEFAULT = new Builder().build();
@@ -58,7 +59,15 @@ public class RequestConfig implements Cloneable {
     private final int connectionRequestTimeout;
     private final int connectTimeout;
     private final int socketTimeout;
-    private final boolean decompressionEnabled;
+    private final boolean contentCompressionEnabled;
+    private final boolean normalizeUri;
+
+    /**
+     * Intended for CDI compatibility
+    */
+    protected RequestConfig() {
+        this(false, null, null, false, null, false, false, false, 0, false, null, null, 0, 0, 0, true, true);
+    }
 
     RequestConfig(
             final boolean expectContinueEnabled,
@@ -76,7 +85,8 @@ public class RequestConfig implements Cloneable {
             final int connectionRequestTimeout,
             final int connectTimeout,
             final int socketTimeout,
-            final boolean decompressionEnabled) {
+            final boolean contentCompressionEnabled,
+            final boolean normalizeUri) {
         super();
         this.expectContinueEnabled = expectContinueEnabled;
         this.proxy = proxy;
@@ -93,7 +103,8 @@ public class RequestConfig implements Cloneable {
         this.connectionRequestTimeout = connectionRequestTimeout;
         this.connectTimeout = connectTimeout;
         this.socketTimeout = socketTimeout;
-        this.decompressionEnabled = decompressionEnabled;
+        this.contentCompressionEnabled = contentCompressionEnabled;
+        this.normalizeUri = normalizeUri;
     }
 
     /**
@@ -305,9 +316,35 @@ public class RequestConfig implements Cloneable {
      * </p>
      *
      * @since 4.4
+     * @deprecated (4.5) Use {@link #isContentCompressionEnabled()}
      */
+    @Deprecated
     public boolean isDecompressionEnabled() {
-        return decompressionEnabled;
+        return contentCompressionEnabled;
+    }
+
+    /**
+     * Determines whether the target server is requested to compress content.
+     * <p>
+     * Default: {@code true}
+     * </p>
+     *
+     * @since 4.5
+     */
+    public boolean isContentCompressionEnabled() {
+        return contentCompressionEnabled;
+    }
+
+    /**
+     * Determines whether client should normalize URIs in requests or not.
+     * <p>
+     * Default: {@code true}
+     * </p>
+     *
+     * @since 4.5.8
+     */
+    public boolean isNormalizeUri() {
+        return normalizeUri;
     }
 
     @Override
@@ -333,7 +370,8 @@ public class RequestConfig implements Cloneable {
         builder.append(", connectionRequestTimeout=").append(connectionRequestTimeout);
         builder.append(", connectTimeout=").append(connectTimeout);
         builder.append(", socketTimeout=").append(socketTimeout);
-        builder.append(", decompressionEnabled=").append(decompressionEnabled);
+        builder.append(", contentCompressionEnabled=").append(contentCompressionEnabled);
+        builder.append(", normalizeUri=").append(normalizeUri);
         builder.append("]");
         return builder.toString();
     }
@@ -360,7 +398,9 @@ public class RequestConfig implements Cloneable {
             .setConnectionRequestTimeout(config.getConnectionRequestTimeout())
             .setConnectTimeout(config.getConnectTimeout())
             .setSocketTimeout(config.getSocketTimeout())
-            .setDecompressionEnabled(config.isDecompressionEnabled());
+            .setDecompressionEnabled(config.isDecompressionEnabled())
+            .setContentCompressionEnabled(config.isContentCompressionEnabled())
+            .setNormalizeUri(config.isNormalizeUri());
     }
 
     public static class Builder {
@@ -380,7 +420,8 @@ public class RequestConfig implements Cloneable {
         private int connectionRequestTimeout;
         private int connectTimeout;
         private int socketTimeout;
-        private boolean decompressionEnabled;
+        private boolean contentCompressionEnabled;
+        private boolean normalizeUri;
 
         Builder() {
             super();
@@ -392,7 +433,8 @@ public class RequestConfig implements Cloneable {
             this.connectionRequestTimeout = -1;
             this.connectTimeout = -1;
             this.socketTimeout = -1;
-            this.decompressionEnabled = true;
+            this.contentCompressionEnabled = true;
+            this.normalizeUri = true;
         }
 
         public Builder setExpectContinueEnabled(final boolean expectContinueEnabled) {
@@ -475,8 +517,23 @@ public class RequestConfig implements Cloneable {
             return this;
         }
 
+        /**
+         * @deprecated (4.5) Set {@link #setContentCompressionEnabled(boolean)} to {@code false} and
+         * add the {@code Accept-Encoding} request header.
+         */
+        @Deprecated
         public Builder setDecompressionEnabled(final boolean decompressionEnabled) {
-            this.decompressionEnabled = decompressionEnabled;
+            this.contentCompressionEnabled = decompressionEnabled;
+            return this;
+        }
+
+        public Builder setContentCompressionEnabled(final boolean contentCompressionEnabled) {
+            this.contentCompressionEnabled = contentCompressionEnabled;
+            return this;
+        }
+
+        public Builder setNormalizeUri(final boolean normalizeUri) {
+            this.normalizeUri = normalizeUri;
             return this;
         }
 
@@ -497,7 +554,8 @@ public class RequestConfig implements Cloneable {
                     connectionRequestTimeout,
                     connectTimeout,
                     socketTimeout,
-                    decompressionEnabled);
+                    contentCompressionEnabled,
+                    normalizeUri);
         }
 
     }
